package AssetAppenderByURL::CMS;

use strict;
use warnings;

sub bulk_save {
    my ( $eh, $app, $objs ) = @_;
    my %param;
    my $q = $app->param;
    my $blog_id = $q->param('blog_id');
    my $blog = MT->model('blog')->load($blog_id);

    my $site_path = $blog->site_path;
    my $site_url = $blog->site_url;

    for my $tmp ( @$objs ) {
        my $obj = $tmp->{ current };
        my %args;
        require MT::Template::Context;
        my $ctx = MT::Template::Context->new;
        $ctx->stash('blog', $blog);
        $ctx->stash('entry', $obj);
        my $text = $ctx->invoke_handler('entrybody', \%args);

        while ( $text =~ /<img src="([^"]+)"/g ) {
            _add_asset_to_entry( $blog_id, $obj->id, $1, $site_path, $site_url );
        }
    }
}

sub post_save {
    my ( $eh, $app, $obj, $orig_obj ) = @_;

    my %param;
    my $q = $app->param;
    my $blog_id = $q->param('blog_id');
    my $blog = MT->model('blog')->load($blog_id);

    my $site_path = $blog->site_path;
    my $site_url = $blog->site_url;

    my %args;
    require MT::Template::Context;
    my $ctx = MT::Template::Context->new;
    $ctx->stash('blog', $blog);
    $ctx->stash('entry', $obj);
    my $text = $ctx->invoke_handler('entrybody', \%args);

    while ( $text =~ /<img src="([^"]+)"/g ) {
        _add_asset_to_entry( $blog_id, $obj->id, $1, $site_path, $site_url );
    }
}

# for cf_image
sub _add_asset_to_entry {
    my ( $blog_id, $id, $url, $site_path, $site_url ) = @_; # no nessesary class_type

    my $asset_id = upload( $blog_id, $url, $site_path, $site_url );
    return if !$asset_id;

    my $obj_asset = MT::ObjectAsset->load({ asset_id => $asset_id, object_ds => 'entry', object_id => $id });
    unless ($obj_asset) {
        my $obj_asset = new MT::ObjectAsset;
        $obj_asset->blog_id($blog_id);
        $obj_asset->asset_id($asset_id);
        $obj_asset->object_ds('entry');
        $obj_asset->object_id($id);
        $obj_asset->save;
    }
}

sub upload {
    my ( $blog_id, $file_url, $site_path, $site_url ) = @_;

    return 0 if !$file_url;

    my $obj = MT->model('asset')->new();
    $obj->blog_id( $blog_id );

    # file URL
    $file_url =~ s/\\/\//g;
    $obj->url( $file_url );

    # file name & label
    ( my $file_name = $file_url ) =~ s/^.*\/([^\/]+)$/$1/;
    $obj->file_name( $file_name );

    # file path
    ( my $url = $file_url ) =~ s/$site_url(.*)/$1/; # path exclude site_url
    $obj->file_path( File::Spec->catfile( $site_path , $url ));

    $obj->label( $url );

    my $asset = MT->model('asset')->load({ class => '*', label => $url });
    return $asset->id if $asset;

    # file ext
    ( my $ext = $file_name ) =~ s/^.*\.([^.]+)$/$1/;
    $obj->file_ext( $ext );

    if ( $ext =~ /jpg|gif|jpe|jpeg|png|bmp|tif|tiff|ico/i) {
        $obj->class('image');
    } elsif ( $ext =~ /mp3|ogg|aif|aiff|wav|wma|aac|flac|m4a/i) {
        $obj->class('audio');
    } elsif ( $ext =~ /mov|avi|3gp|asf|mp4|qt|wmv|asx|mpg|flv|mkv|ogm/i) {
        $obj->class('video');
    } else {
        $obj->class('file');
    }

    my $mime_type = 'image/' . lc $ext if $obj->class eq 'image';
    $obj->mime_type( $mime_type );

    $obj->save or die $obj->errstr;
    return $obj->id;
}

1;
